<?php
// 日志记录配置
define('PRODUCT_DEBUG_ENABLED', false); // 设置为false将禁用详细日志记录

// 确保日志目录和文件存在
$logDir = __DIR__ . '/../../logs';
$logFile = $logDir . '/product_debug.log';

// 自定义日志函数
function logDebug($message) {
    // 如果日志已禁用，则直接返回
    if (!PRODUCT_DEBUG_ENABLED) {
        return;
    }
    
    global $logFile, $logDir;
    
    // 防止日志错误影响网站正常运行
    try {
        // 如果目录不存在且无法创建，则静默失败
        if (!file_exists($logDir)) {
            if (!@mkdir($logDir, 0777, true)) {
                // 使用PHP的内置错误日志作为备用
                @error_log("产品模块: " . $message);
                return; // 无法创建目录则退出
            }
        }
        
        // 检查日志文件是否可写
        if (is_writable($logDir) || (file_exists($logFile) && is_writable($logFile))) {
            $timestamp = date('Y-m-d H:i:s');
            @file_put_contents($logFile, "[$timestamp] $message\n", FILE_APPEND);
        } else {
            // 使用PHP的内置错误日志作为备用
            @error_log("产品模块: " . $message);
        }
    } catch (Exception $e) {
        // 捕获任何异常但不做任何处理，防止日志错误影响页面
        // 尝试使用PHP的内置错误日志
        @error_log("产品模块异常: " . $message . " - " . $e->getMessage());
    }
}

// 记录日志（现在不会因权限问题导致错误）
@logDebug("开始加载product_data.php");

// 确保已经包含必要的配置文件
if (!isset($GLOBALS['Telegram'])) {
    include_once __DIR__ . '/../../public/config.php';
    @logDebug("包含了config.php文件");
}

// 包含数据库连接
require_once __DIR__ . '/../../config/database.php';
@logDebug("包含了database.php文件");

// 确保有数据库连接
global $db;
if (!isset($db) || !$db) {
    $db = getDbConnection();
    if (!$db) {
        @logDebug("错误: 无法创建数据库连接");
    } else {
        @logDebug("数据库连接成功创建");
    }
} else {
    @logDebug("数据库连接已存在");
}

// 测试数据库连接是否有效
try {
    $stmt = $db->query("SELECT 1");
    @logDebug("数据库连接测试成功");
} catch (Exception $e) {
    @logDebug("数据库连接测试失败: " . $e->getMessage());
}

// 提取产品数据的函数
function getProducts($filters = [], $sort = 'recommended', $order = 'asc', $page = 1, $limit = 10) {
    // 从数据库获取产品数据
    try {
        global $db;
        
        if (function_exists('logDebug')) {
            @logDebug("开始查询产品数据: " . print_r($filters, true));
        }
        
        // 检查数据库连接是否成功
        if (!$db) {
            if (function_exists('logDebug')) {
                @logDebug("数据库连接失败");
            }
            return [
                'products' => [],
                'total' => 0,
                'page' => $page,
                'limit' => $limit,
                'error' => '数据库连接失败'
            ];
        }
        
        // 构建查询条件
        $whereClause = "";
        $params = [];
        $conditions = [];
        
        // 根据筛选条件构建WHERE子句
        if (!empty($filters)) {
            // 标签筛选 - 需要检查tags字段中是否包含指定的标签ID
            if (isset($filters['tag']) && $filters['tag'] > 0) {
                // 标签格式是逗号分隔的数字，如: "8,15,18,16,22,21,3"
                // 需要查找tags字段中包含特定标签ID的记录
                $tagId = (int)$filters['tag'];
                // 构建LIKE条件，考虑开头、中间、结尾的情况
                $conditions[] = "(p.tags LIKE :tag_start OR p.tags LIKE :tag_middle OR p.tags LIKE :tag_end OR p.tags = :tag_exact)";
                $params[':tag_start'] = $tagId . ',%';
                $params[':tag_middle'] = '%,' . $tagId . ',%';
                $params[':tag_end'] = '%,' . $tagId;
                $params[':tag_exact'] = (string)$tagId;
                
                if (function_exists('logDebug')) {
                    @logDebug("添加标签筛选条件: tag_id = " . $tagId);
                }
            }
            
            // 地区筛选 - 根据area表的结构调整查询逻辑
            if (isset($filters['region']) && $filters['region'] > 0) {
                // 直接使用region_id筛选特定地区
                $conditions[] = "p.region_id = :region_id";
                $params[':region_id'] = (int)$filters['region'];
                if (function_exists('logDebug')) {
                    @logDebug("添加地区筛选条件: region_id = " . $filters['region']);
                }
            } elseif (isset($filters['parent_region']) && $filters['parent_region'] > 0) {
                // 对于父级地区，需要查找该地区下所有子地区的产品
                // 首先查询area表获取该父级地区下所有子地区的ID
                try {
                    // 查询子地区ID
                    $childRegionIds = [];
                    $childRegionsSql = "SELECT id FROM area WHERE parent_id = :parent_id AND status = 1";
                    $childRegionsStmt = $db->prepare($childRegionsSql);
                    $childRegionsStmt->bindValue(':parent_id', (int)$filters['parent_region'], PDO::PARAM_INT);
                    $childRegionsStmt->execute();
                    $childRegions = $childRegionsStmt->fetchAll(PDO::FETCH_COLUMN);
                    
                    if (function_exists('logDebug')) {
                        @logDebug("查询到子地区: " . implode(',', $childRegions));
                    }
                    
                    // 如果找到子地区，则使用IN条件进行筛选
                    if (!empty($childRegions)) {
                        // 准备IN查询的占位符
                        $placeholders = [];
                        foreach ($childRegions as $index => $id) {
                            $placeholder = ":region_" . $index;
                            $placeholders[] = $placeholder;
                            $params[$placeholder] = $id;
                        }
                        
                        $inPlaceholders = implode(',', $placeholders);
                        $conditions[] = "p.region_id IN ($inPlaceholders)";
                        
                        if (function_exists('logDebug')) {
                            @logDebug("添加父地区筛选条件: parent_id = " . $filters['parent_region'] . ", 子地区: " . implode(',', $childRegions));
                        }
                    } else {
                        if (function_exists('logDebug')) {
                            @logDebug("未找到子地区，父地区ID: " . $filters['parent_region']);
                        }
                    }
                } catch (Exception $e) {
                    // 如果查询子地区失败，记录错误
                    if (function_exists('logDebug')) {
                        @logDebug("查询子地区失败: " . $e->getMessage());
                    }
                }
            }
            
            // 价格范围筛选
            if (isset($filters['min_price']) && $filters['min_price'] !== null) {
                $conditions[] = "p.price >= :min_price";
                $params[':min_price'] = $filters['min_price'];
                if (function_exists('logDebug')) {
                    @logDebug("添加最低价格筛选条件: min_price = " . $filters['min_price']);
                }
            }
            if (isset($filters['max_price']) && $filters['max_price'] !== null) {
                $conditions[] = "p.price <= :max_price";
                $params[':max_price'] = $filters['max_price'];
                if (function_exists('logDebug')) {
                    @logDebug("添加最高价格筛选条件: max_price = " . $filters['max_price']);
                }
            }
        }
        
        // 只查询状态为启用的产品
        $conditions[] = "p.status = 1";
        
        // 组合WHERE子句
        if (!empty($conditions)) {
            $whereClause = "WHERE " . implode(" AND ", $conditions);
        }
        
        // 排序条件
        $orderClause = "ORDER BY p.id DESC"; // 默认排序
        
        // 根据排序参数设置排序规则
        if ($sort == 'price') {
            $orderDirection = ($order == 'desc') ? 'DESC' : 'ASC';
            $orderClause = "ORDER BY p.price $orderDirection";
            if (function_exists('logDebug')) {
                @logDebug("设置价格排序: " . $orderDirection);
            }
        } elseif ($sort == 'created') {
            $orderDirection = ($order == 'asc') ? 'ASC' : 'DESC';
            $orderClause = "ORDER BY p.created_at $orderDirection";
            if (function_exists('logDebug')) {
                @logDebug("设置创建时间排序: " . $orderDirection);
            }
        }
        
        // 分页
        $offset = ($page - 1) * $limit;
        if (function_exists('logDebug')) {
            @logDebug("设置分页: 页码=$page, 每页显示=$limit, 偏移量=$offset");
        }
        
        // 构建SQL查询
        $sql = "SELECT p.* FROM products p $whereClause $orderClause LIMIT :offset, :limit";
        if (function_exists('logDebug')) {
            @logDebug("SQL查询: $sql");
            @logDebug("绑定参数: " . print_r($params, true));
        }
        
        $stmt = $db->prepare($sql);
        
        // 绑定参数
        foreach ($params as $param => $value) {
            if (is_int($param)) {
                // 对于IN条件的参数，使用bindValue和PDO::PARAM_INT
                $stmt->bindValue($param, $value, PDO::PARAM_INT);
            } else {
                $stmt->bindValue($param, $value);
            }
        }
        $stmt->bindValue(':offset', $offset, PDO::PARAM_INT);
        $stmt->bindValue(':limit', $limit, PDO::PARAM_INT);
        
        // 执行查询
        $stmt->execute();
        $products = $stmt->fetchAll(PDO::FETCH_ASSOC);
        if (function_exists('logDebug')) {
            @logDebug("查询结果数量: " . count($products));
        }
        
        // 获取总数
        // 需要修改计数SQL以移除LIMIT子句
        $countSql = "SELECT COUNT(*) FROM products p $whereClause";
        if (function_exists('logDebug')) {
            @logDebug("计数SQL: $countSql");
        }
        
        $countStmt = $db->prepare($countSql);
        foreach ($params as $param => $value) {
            if (is_int($param)) {
                // 对于IN条件的参数，使用bindValue和PDO::PARAM_INT
                $countStmt->bindValue($param, $value, PDO::PARAM_INT);
            } else {
                $countStmt->bindValue($param, $value);
            }
        }
        $countStmt->execute();
        $totalCount = $countStmt->fetchColumn();
        if (function_exists('logDebug')) {
            @logDebug("总记录数: $totalCount");
        }
        
        if (empty($products)) {
            if (function_exists('logDebug')) {
                @logDebug("未找到符合条件的产品记录");
            }
        } else {
            if (function_exists('logDebug')) {
                @logDebug("产品详情示例: " . print_r($products[0], true));
            }
        }
        
        return [
            'products' => $products,
            'total' => $totalCount,
            'page' => $page,
            'limit' => $limit
        ];
    } catch (Exception $e) {
        // 发生错误时记录错误信息并返回空数组
        if (function_exists('logDebug')) {
            @logDebug("产品查询错误: " . $e->getMessage());
            @logDebug("错误堆栈: " . $e->getTraceAsString());
        }
        
        return [
            'products' => [],
            'total' => 0,
            'page' => $page,
            'limit' => $limit,
            'error' => $e->getMessage()
        ];
    }
}

// 获取地区数据
function getRegionData() {
    // 从数据库获取地区数据，而不是使用硬编码数据
    global $db;
    $result = [
        'parentRegions' => [],
        'childRegionsByParent' => []
    ];
    
    try {
        // 获取一级地区（parent_id为null或0的记录）
        $parentQuery = "SELECT id, name, slug FROM area WHERE (parent_id IS NULL OR parent_id = 0) AND status = 1 ORDER BY sort_order ASC";
        $parentStmt = $db->query($parentQuery);
        $result['parentRegions'] = $parentStmt->fetchAll();
        
        // 获取所有子地区
        $childrenQuery = "SELECT id, name, slug, parent_id FROM area WHERE parent_id IS NOT NULL AND parent_id != 0 AND status = 1 ORDER BY sort_order ASC";
        $childrenStmt = $db->query($childrenQuery);
        $children = $childrenStmt->fetchAll();
        
        // 按父级ID组织子地区
        $childRegionsByParent = [];
        foreach ($children as $child) {
            $parentId = $child['parent_id'];
            if (!isset($childRegionsByParent[$parentId])) {
                $childRegionsByParent[$parentId] = [];
            }
            $childRegionsByParent[$parentId][] = $child;
        }
        $result['childRegionsByParent'] = $childRegionsByParent;
        
        if (function_exists('logDebug')) {
            @logDebug("从数据库获取地区数据：" . count($result['parentRegions']) . " 父地区, " . count($result['childRegionsByParent']) . " 个父地区有子地区");
        }
    } catch (Exception $e) {
        // 发生错误时记录并返回空数据
        if (function_exists('logDebug')) {
            @logDebug("获取地区数据错误: " . $e->getMessage());
        }
    }
    
    return $result;
}

// 获取产品标签
function getTagData() {
    // 从数据库获取标签数据，而不是使用硬编码数据
    global $db;
    $result = [
        'parentTags' => [],
        'childTagsByParent' => []
    ];
    
    try {
        // 获取一级标签（parent_id为0的记录）
        $parentQuery = "SELECT id, name, slug FROM tags WHERE parent_id = 0 AND status = 1 ORDER BY sort_order ASC";
        $parentStmt = $db->query($parentQuery);
        $result['parentTags'] = $parentStmt->fetchAll();
        
        // 获取所有子标签
        $childrenQuery = "SELECT id, name, slug, parent_id FROM tags WHERE parent_id != 0 AND status = 1 ORDER BY sort_order ASC";
        $childrenStmt = $db->query($childrenQuery);
        $children = $childrenStmt->fetchAll();
        
        // 按父级ID组织子标签
        $childTagsByParent = [];
        foreach ($children as $child) {
            $parentId = $child['parent_id'];
            if (!isset($childTagsByParent[$parentId])) {
                $childTagsByParent[$parentId] = [];
            }
            $childTagsByParent[$parentId][] = $child;
        }
        $result['childTagsByParent'] = $childTagsByParent;
        
        if (function_exists('logDebug')) {
            @logDebug("从数据库获取标签数据：" . count($result['parentTags']) . " 父标签, " . count($result['childTagsByParent']) . " 个父标签有子标签");
        }
    } catch (Exception $e) {
        // 发生错误时记录并返回空数据
        if (function_exists('logDebug')) {
            @logDebug("获取标签数据错误: " . $e->getMessage());
        }
    }
    
    return $result;
}

// 获取价格范围数据
function getPriceRangeData() {
    return [
        ['min' => null, 'max' => null, 'label' => '全部'],
        ['min' => 0, 'max' => 500, 'label' => '0-500元'],
        ['min' => 500, 'max' => 1000, 'label' => '500-1000元'],
        ['min' => 1000, 'max' => 2000, 'label' => '1000-2000元'],
        ['min' => 2000, 'max' => null, 'label' => '2000元以上']
    ];
}

// 获取线路数据
function getLineData() {
    // 实际项目中，应该从数据库获取线路数据
    return [
        1 => '电信',
        2 => '联通',
        3 => '移动',
        4 => 'CN2',
        5 => 'BGP'
    ];
}

// 辅助函数
function generateProductTitle($product, $areaNames, $allRegions, $parentRegions) {
    // 根据产品数据生成标题
    $region = isset($areaNames[$product['region_id']]) ? $areaNames[$product['region_id']] : '未知地区';
    return htmlspecialchars($region . ' ' . ($product['title'] ?? '服务器'));
}

function getRegionDisplay($regionId, $areaNames) {
    return isset($areaNames[$regionId]) ? $areaNames[$regionId] : '未知地区';
}

function getLineDisplay($lineId, $lineData) {
    return isset($lineData[$lineId]) ? $lineData[$lineId] : '未知线路';
}

function getDefenseDisplay($defense) {
    if (!$defense) return '无防御';
    return htmlspecialchars($defense);
}

function getTrafficDisplay($traffic) {
    if (!$traffic) return '无限制';
    return htmlspecialchars($traffic);
}

function getPriceDisplay($price, $originalPrice = null) {
    $priceText = number_format($price, 2) . '元/月';
    if ($originalPrice && $originalPrice > $price) {
        $priceText = '<span class="current-price">' . $priceText . '</span> <span class="original-price">' . number_format($originalPrice, 2) . '元/月</span>';
    }
    return $priceText;
}

function formatProductId($id) {
    return 'SKU-' . str_pad($id, 6, '0', STR_PAD_LEFT);
}

function generatePagination($currentPage, $totalPages, $urlPattern = '/product/?page=%d') {
    if ($totalPages <= 1) return '';
    
    $html = '<div class="pagination">';
    
    // 上一页按钮
    if ($currentPage > 1) {
        $html .= '<a href="' . sprintf($urlPattern, $currentPage - 1) . '" class="page-nav prev">上一页</a>';
    } else {
        $html .= '<span class="page-nav prev disabled">上一页</span>';
    }
    
    // 页码按钮
    $startPage = max(1, $currentPage - 2);
    $endPage = min($totalPages, $startPage + 4);
    
    if ($startPage > 1) {
        $html .= '<a href="' . sprintf($urlPattern, 1) . '" class="page-number">1</a>';
        if ($startPage > 2) {
            $html .= '<span class="page-ellipsis">...</span>';
        }
    }
    
    for ($i = $startPage; $i <= $endPage; $i++) {
        if ($i == $currentPage) {
            $html .= '<span class="page-number active">' . $i . '</span>';
        } else {
            $html .= '<a href="' . sprintf($urlPattern, $i) . '" class="page-number">' . $i . '</a>';
        }
    }
    
    if ($endPage < $totalPages) {
        if ($endPage < $totalPages - 1) {
            $html .= '<span class="page-ellipsis">...</span>';
        }
        $html .= '<a href="' . sprintf($urlPattern, $totalPages) . '" class="page-number">' . $totalPages . '</a>';
    }
    
    // 下一页按钮
    if ($currentPage < $totalPages) {
        $html .= '<a href="' . sprintf($urlPattern, $currentPage + 1) . '" class="page-nav next">下一页</a>';
    } else {
        $html .= '<span class="page-nav next disabled">下一页</span>';
    }
    
    $html .= '</div>';
    
    return $html;
} 
